void drawTitle() {
    textSize(24);
    fill(FONT_COLOR);
    textAlign(CENTER, CENTER);
    text("Items acquired over time", width/2, MARGIN_TOP / 2);
}

void drawLegend() {
    textSize(14);
    fill(FONT_COLOR);
    textAlign(RIGHT, CENTER);
    float eachHeight = (height - MARGIN_TOP * 2) / 10;
    // draw dewey class label
    for (int i = 0; i < DEWEY_CLASS.length; i++) {
        text(DEWEY_CLASS[i], MARGIN_LEFT - 10, i * eachHeight + MARGIN_TOP + eachHeight / 2);
    }
    // draw separate line
    stroke(FONT_COLOR);
    for (int i = 1; i < DEWEY_CLASS.length; i++) {
        line(MARGIN_LEFT-90, i * 100 * cellHeight + MARGIN_TOP, MARGIN_LEFT - 10, i * 100 * cellHeight + MARGIN_TOP);
    }

    // draw book color legend
    fill(BOOK_COLOR);
    noStroke();
    float squareWidth = 20;
    float bookX = MARGIN_LEFT + 50 - squareWidth / 2;
    float bookY = height - MARGIN_TOP / 2 - squareWidth / 2;
    square(bookX, bookY, squareWidth);
    fill(FONT_COLOR);
    text("book", bookX + 60, bookY + squareWidth / 2);
    // draw media color legend
    fill(MEDIA_COLOR);
    float mediaX = bookX + 80;
    float mediaY = bookY;
    square(mediaX, mediaY, squareWidth);
    fill(FONT_COLOR);
    text("media", mediaX + 65, mediaY + squareWidth / 2);
    // draw misc color legend
    fill(MISC_COLOR);
    float miscX = mediaX + 80;
    float miscY = mediaY;
    square(miscX, miscY, squareWidth);
    fill(FONT_COLOR);
    text("misc", miscX + 55, miscY + squareWidth / 2);

    // draw instruction
    textAlign(RIGHT, CENTER);
    text("press 1, 2, 3 and space key to change graph, move mouse over circles to view dewey class and title.", width - 20, height - MARGIN_TOP / 2);
}

void drawOverview() {
    // draw overview graph
    pgOverview.beginDraw();
    pgOverview.background(BG_COLOR);
    pgOverview.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("book")) {
            pgOverview.fill(BOOK_COLOR);
        } else if (e.itemType.equals("media")) {
            continue;
        } else if (e.itemType.equals("misc")) {
            continue;
        }
        pgOverview.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
    }
    // draw media second, so that it shows on top
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("media")) {
            pgOverview.fill(MEDIA_COLOR);
            pgOverview.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    // draw misc last, so that it shows on top
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("misc")) {
            pgOverview.fill(MISC_COLOR);
            pgOverview.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgOverview.endDraw();
    println("Finished drawing overview!");
    loaded = true;
}

void drawBook() {
    // draw book graph
    pgBook.beginDraw();
    pgBook.background(BG_COLOR);
    pgBook.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("book")) {
            pgBook.fill(BOOK_COLOR);
            pgBook.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgBook.endDraw();
    println("Finished drawing book!");
}

void drawMedia() {
    // draw media graph
    pgMedia.beginDraw();
    pgMedia.background(BG_COLOR);
    pgMedia.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("media")) {
            pgMedia.fill(MEDIA_COLOR);
            pgMedia.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgMedia.endDraw();
    println("Finished drawing media!");
}

void drawMisc() {
    // draw misc graph
    pgMisc.beginDraw();
    pgMisc.background(BG_COLOR);
    pgMisc.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("misc")) {
            pgMisc.fill(MISC_COLOR);
            pgMisc.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgMisc.endDraw();
    println("Finished drawing misc!");
}

String getHoverText(float x, float y) {
    int itemNumber = (int)(x * cellWidth + itemNumberStart);
    int deweyClass = (int)(y / cellHeight);
    for (int i = itemNumber; i >= itemNumber - cellWidth; i--) {
        if (map.containsKey(i)) {
            Entry e = map.get(i);
            // get a close estimate
            if (abs(e.deweyClass - deweyClass) > 10) {
                continue;
            }
            switch (keyState) {
                case OVERVIEW:
                    return String.valueOf(e.deweyClass) + "," + e.title;
                case BOOK:
                    if (e.itemType.equals("book")) {
                        return String.valueOf(e.deweyClass) + "," + e.title;
                    }
                case MEDIA:
                    if (e.itemType.equals("media")) {
                        return String.valueOf(e.deweyClass) + "," + e.title;
                    }
                case MISC:
                    if (e.itemType.equals("misc")) {
                        return String.valueOf(e.deweyClass) + "," + e.title;
                    }
            }
        }
    }
    return "";
}

void keyPressed() {
    switch (key) {
        case '1':
            keyState = KeyState.BOOK;
            break;
        case '2':
            keyState = KeyState.MEDIA;
            break;
        case '3':
            keyState = KeyState.MISC;
            break;
        case ' ':
            keyState = KeyState.OVERVIEW;
            break;
    }
}

void mouseMotion() {
    int mX = mouseX, mY = mouseY;
    if (mX > MARGIN_LEFT && mY > MARGIN_TOP && mY < (height - MARGIN_TOP)) {
        hoverText = getHoverText(mX - MARGIN_LEFT, mY - MARGIN_TOP);
        textSize(14);
        // if it's on right edge, draw text on left side
        if ((width - mX) < textWidth(hoverText) && (mX - MARGIN_LEFT) > textWidth(hoverText)) {
            textAlign(RIGHT, LEFT);
        } else {
            textAlign(TOP, LEFT);
        }
        // if it's on upper edge, draw text on lower corner
        if ((mY - MARGIN_TOP) < 15) {
            text(hoverText, mX+13, mY+13);
        } else {
            text(hoverText, mX-2, mY-2);
        }
    }
}
