import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class time_series extends PApplet {

/********************************************************
 * Data Visualization - Items acquired over time        *
 * Tested in Processing 3.5.3                           *
 *                                                      *
 * Author: Jiaheng Tang                                 *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Visualizing the dewey classes of the items  *
 *          acquired by Seattle Public Library over     *
 *          time                                        *
 *                                                      *
 * Usage:   Press '1', '2', '3' to view Book,           *
 *          Media, and Misc graph                       *
 *          Press space to view Overview graph          *
 *          Move mouse over dots to view dewey class    *
 *          and title                                   *
 ********************************************************/



Table table;
int rows, cols;
List<Entry> entrys;
HashMap<Integer, Entry> map;
final float MARGIN_TOP = 40;
final float MARGIN_LEFT = 160;
final float NUM_OF_DEWEY_CLASS = 999;  // from 001 to 999
final float CIRCLE_SIZE = 1.0f;
float cellWidth, cellHeight;
final int BG_COLOR = color(0, 0, 0);
final int BOOK_COLOR = color(221, 91, 73);
final int MEDIA_COLOR = color(225, 226, 114);
final int MISC_COLOR = color(81, 97, 175);
final int FONT_COLOR = color(255, 255, 255);
PGraphics pgOverview, pgBook, pgMedia, pgMisc;
enum KeyState {
    OVERVIEW, BOOK, MEDIA, MISC
}
KeyState keyState;
String hoverText = "";
boolean loaded = false;

final String[] DEWEY_CLASS = {"000 - 099\nComputer Science,\n Information\n& General works",
                        "100 - 199\nPhilosophy\n& Psychology",
                        "200 - 299\nReligon",
                        "300 - 399\nSocial Sciences",
                        "400 - 499\nLanguage",
                        "500 - 599\nScience",
                        "600 - 699\nTechnology",
                        "700 - 799\nArts & Recreation",
                        "800 - 899\nLiterature",
                        "900 - 999\nHistory & Geography"};

float itemNumberStart;

public void setup() {
    
    // load data
    table = loadTable("out_processed.csv", "header");
    rows = table.getRowCount();
    cols = table.getColumnCount();
    entrys = new ArrayList<Entry>();
    map = new HashMap<Integer, Entry>();
    println("Rows: " + rows);
    for (int i = 0; i < rows; i++) {
        float itemNumber = table.getFloat(i, 0);
        int bibNumber = table.getInt(i, 1);
        String itemType = table.getString(i, 2);
        String title = table.getString(i, 3);
        int deweyClass = table.getInt(i, 4);

        // check duplicate itemNumber
        if (entrys.size() > 0 && entrys.get(entrys.size() - 1).itemNumber == itemNumber){
            continue;
        }
        Entry entry = new Entry(itemNumber, bibNumber, itemType, title, deweyClass);
        entrys.add(entry);
        map.put((int)itemNumber, entry);
    }
    println("Finished loading " + entrys.size() + " entries!");
    itemNumberStart = entrys.get(0).itemNumber;
    cellWidth = (entrys.get(entrys.size() - 1).itemNumber - itemNumberStart) / (width - MARGIN_LEFT);
    cellHeight = (height - MARGIN_TOP * 2) / NUM_OF_DEWEY_CLASS;
    background(BG_COLOR);

    // draw graphs in separate threads
    pgOverview = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    pgBook = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    pgMedia = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    pgMisc = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    keyState = KeyState.OVERVIEW;
    thread("drawOverview");
    thread("drawBook");
    thread("drawMedia");
    thread("drawMisc");

    drawTitle();
    drawLegend();
}

public void draw() {
    // check keystate to show graphs
    switch (keyState) {
        case OVERVIEW:
            image(pgOverview, MARGIN_LEFT, MARGIN_TOP);
            break;
        case BOOK:
            image(pgBook, MARGIN_LEFT, MARGIN_TOP);
            break;
        case MEDIA:
            image(pgMedia, MARGIN_LEFT, MARGIN_TOP);
            break;
        case MISC:
            image(pgMisc, MARGIN_LEFT, MARGIN_TOP);
            break;
    }
    if (loaded){
        mouseMotion();
    }
}
class Entry {
    float itemNumber;
    int bibNumber;
    String itemType;
    String title;
    int deweyClass;

    Entry(float i, int b, String it, String t, int d){
        this.itemNumber = i;
        this.bibNumber = b;
        this.itemType = it;
        this.title = t;
        this.deweyClass = d;
    }
}
public void drawTitle() {
    textSize(24);
    fill(FONT_COLOR);
    textAlign(CENTER, CENTER);
    text("Items acquired over time", width/2, MARGIN_TOP / 2);
}

public void drawLegend() {
    textSize(14);
    fill(FONT_COLOR);
    textAlign(RIGHT, CENTER);
    float eachHeight = (height - MARGIN_TOP * 2) / 10;
    // draw dewey class label
    for (int i = 0; i < DEWEY_CLASS.length; i++) {
        text(DEWEY_CLASS[i], MARGIN_LEFT - 10, i * eachHeight + MARGIN_TOP + eachHeight / 2);
    }
    // draw separate line
    stroke(FONT_COLOR);
    for (int i = 1; i < DEWEY_CLASS.length; i++) {
        line(MARGIN_LEFT-90, i * 100 * cellHeight + MARGIN_TOP, MARGIN_LEFT - 10, i * 100 * cellHeight + MARGIN_TOP);
    }

    // draw book color legend
    fill(BOOK_COLOR);
    noStroke();
    float squareWidth = 20;
    float bookX = MARGIN_LEFT + 50 - squareWidth / 2;
    float bookY = height - MARGIN_TOP / 2 - squareWidth / 2;
    square(bookX, bookY, squareWidth);
    fill(FONT_COLOR);
    text("book", bookX + 60, bookY + squareWidth / 2);
    // draw media color legend
    fill(MEDIA_COLOR);
    float mediaX = bookX + 80;
    float mediaY = bookY;
    square(mediaX, mediaY, squareWidth);
    fill(FONT_COLOR);
    text("media", mediaX + 65, mediaY + squareWidth / 2);
    // draw misc color legend
    fill(MISC_COLOR);
    float miscX = mediaX + 80;
    float miscY = mediaY;
    square(miscX, miscY, squareWidth);
    fill(FONT_COLOR);
    text("misc", miscX + 55, miscY + squareWidth / 2);

    // draw instruction
    textAlign(RIGHT, CENTER);
    text("press 1, 2, 3 and space key to change graph, move mouse over circles to view dewey class and title.", width - 20, height - MARGIN_TOP / 2);
}

public void drawOverview() {
    // draw overview graph
    pgOverview.beginDraw();
    pgOverview.background(BG_COLOR);
    pgOverview.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("book")) {
            pgOverview.fill(BOOK_COLOR);
        } else if (e.itemType.equals("media")) {
            continue;
        } else if (e.itemType.equals("misc")) {
            continue;
        }
        pgOverview.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
    }
    // draw media second, so that it shows on top
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("media")) {
            pgOverview.fill(MEDIA_COLOR);
            pgOverview.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    // draw misc last, so that it shows on top
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("misc")) {
            pgOverview.fill(MISC_COLOR);
            pgOverview.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgOverview.endDraw();
    println("Finished drawing overview!");
    loaded = true;
}

public void drawBook() {
    // draw book graph
    pgBook.beginDraw();
    pgBook.background(BG_COLOR);
    pgBook.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("book")) {
            pgBook.fill(BOOK_COLOR);
            pgBook.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgBook.endDraw();
    println("Finished drawing book!");
}

public void drawMedia() {
    // draw media graph
    pgMedia.beginDraw();
    pgMedia.background(BG_COLOR);
    pgMedia.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("media")) {
            pgMedia.fill(MEDIA_COLOR);
            pgMedia.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgMedia.endDraw();
    println("Finished drawing media!");
}

public void drawMisc() {
    // draw misc graph
    pgMisc.beginDraw();
    pgMisc.background(BG_COLOR);
    pgMisc.noStroke();
    for (int i = 0; i < entrys.size(); i++) {
        Entry e = entrys.get(i);
        if (e.itemType.equals("misc")) {
            pgMisc.fill(MISC_COLOR);
            pgMisc.circle((e.itemNumber - itemNumberStart) / cellWidth, e.deweyClass * cellHeight, CIRCLE_SIZE);
        }
    }
    pgMisc.endDraw();
    println("Finished drawing misc!");
}

public String getHoverText(float x, float y) {
    int itemNumber = (int)(x * cellWidth + itemNumberStart);
    int deweyClass = (int)(y / cellHeight);
    for (int i = itemNumber; i >= itemNumber - cellWidth; i--) {
        if (map.containsKey(i)) {
            Entry e = map.get(i);
            // get a close estimate
            if (abs(e.deweyClass - deweyClass) > 10) {
                continue;
            }
            switch (keyState) {
                case OVERVIEW:
                    return String.valueOf(e.deweyClass) + "," + e.title;
                case BOOK:
                    if (e.itemType.equals("book")) {
                        return String.valueOf(e.deweyClass) + "," + e.title;
                    }
                case MEDIA:
                    if (e.itemType.equals("media")) {
                        return String.valueOf(e.deweyClass) + "," + e.title;
                    }
                case MISC:
                    if (e.itemType.equals("misc")) {
                        return String.valueOf(e.deweyClass) + "," + e.title;
                    }
            }
        }
    }
    return "";
}

public void keyPressed() {
    switch (key) {
        case '1':
            keyState = KeyState.BOOK;
            break;
        case '2':
            keyState = KeyState.MEDIA;
            break;
        case '3':
            keyState = KeyState.MISC;
            break;
        case ' ':
            keyState = KeyState.OVERVIEW;
            break;
    }
}

public void mouseMotion() {
    int mX = mouseX, mY = mouseY;
    if (mX > MARGIN_LEFT && mY > MARGIN_TOP && mY < (height - MARGIN_TOP)) {
        hoverText = getHoverText(mX - MARGIN_LEFT, mY - MARGIN_TOP);
        textSize(14);
        // if it's on right edge, draw text on left side
        if ((width - mX) < textWidth(hoverText) && (mX - MARGIN_LEFT) > textWidth(hoverText)) {
            textAlign(RIGHT, LEFT);
        } else {
            textAlign(TOP, LEFT);
        }
        // if it's on upper edge, draw text on lower corner
        if ((mY - MARGIN_TOP) < 15) {
            text(hoverText, mX+13, mY+13);
        } else {
            text(hoverText, mX-2, mY-2);
        }
    }
}
  public void settings() {  fullScreen(); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "time_series" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
