/********************************************************
 * Data Visualization - Items acquired over time        *
 * Tested in Processing 3.5.3                           *
 *                                                      *
 * Author: Jiaheng Tang                                 *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Visualizing the dewey classes of the items  *
 *          acquired by Seattle Public Library over     *
 *          time                                        *
 *                                                      *
 * Usage:   Press '1', '2', '3' to view Book,           *
 *          Media, and Misc graph                       *
 *          Press space to view Overview graph          *
 *          Move mouse over dots to view dewey class    *
 *          and title                                   *
 ********************************************************/

import java.util.*;

Table table;
int rows, cols;
List<Entry> entrys;
HashMap<Integer, Entry> map;
final float MARGIN_TOP = 40;
final float MARGIN_LEFT = 160;
final float NUM_OF_DEWEY_CLASS = 999;  // from 001 to 999
final float CIRCLE_SIZE = 1.0;
float cellWidth, cellHeight;
final color BG_COLOR = color(0, 0, 0);
final color BOOK_COLOR = color(221, 91, 73);
final color MEDIA_COLOR = color(225, 226, 114);
final color MISC_COLOR = color(81, 97, 175);
final color FONT_COLOR = color(255, 255, 255);
PGraphics pgOverview, pgBook, pgMedia, pgMisc;
enum KeyState {
    OVERVIEW, BOOK, MEDIA, MISC
}
KeyState keyState;
String hoverText = "";
boolean loaded = false;

final String[] DEWEY_CLASS = {"000 - 099\nComputer Science,\n Information\n& General works",
                        "100 - 199\nPhilosophy\n& Psychology",
                        "200 - 299\nReligon",
                        "300 - 399\nSocial Sciences",
                        "400 - 499\nLanguage",
                        "500 - 599\nScience",
                        "600 - 699\nTechnology",
                        "700 - 799\nArts & Recreation",
                        "800 - 899\nLiterature",
                        "900 - 999\nHistory & Geography"};

float itemNumberStart;

void setup() {
    fullScreen();
    // load data
    table = loadTable("out_processed.csv", "header");
    rows = table.getRowCount();
    cols = table.getColumnCount();
    entrys = new ArrayList<Entry>();
    map = new HashMap<Integer, Entry>();
    println("Rows: " + rows);
    for (int i = 0; i < rows; i++) {
        float itemNumber = table.getFloat(i, 0);
        int bibNumber = table.getInt(i, 1);
        String itemType = table.getString(i, 2);
        String title = table.getString(i, 3);
        int deweyClass = table.getInt(i, 4);

        // check duplicate itemNumber
        if (entrys.size() > 0 && entrys.get(entrys.size() - 1).itemNumber == itemNumber){
            continue;
        }
        Entry entry = new Entry(itemNumber, bibNumber, itemType, title, deweyClass);
        entrys.add(entry);
        map.put((int)itemNumber, entry);
    }
    println("Finished loading " + entrys.size() + " entries!");
    itemNumberStart = entrys.get(0).itemNumber;
    cellWidth = (entrys.get(entrys.size() - 1).itemNumber - itemNumberStart) / (width - MARGIN_LEFT);
    cellHeight = (height - MARGIN_TOP * 2) / NUM_OF_DEWEY_CLASS;
    background(BG_COLOR);

    // draw graphs in separate threads
    pgOverview = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    pgBook = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    pgMedia = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    pgMisc = createGraphics((int)(width - MARGIN_LEFT + 10), (int)(height - MARGIN_TOP * 2 + 10));
    keyState = KeyState.OVERVIEW;
    thread("drawOverview");
    thread("drawBook");
    thread("drawMedia");
    thread("drawMisc");

    drawTitle();
    drawLegend();
}

void draw() {
    // check keystate to show graphs
    switch (keyState) {
        case OVERVIEW:
            image(pgOverview, MARGIN_LEFT, MARGIN_TOP);
            break;
        case BOOK:
            image(pgBook, MARGIN_LEFT, MARGIN_TOP);
            break;
        case MEDIA:
            image(pgMedia, MARGIN_LEFT, MARGIN_TOP);
            break;
        case MISC:
            image(pgMisc, MARGIN_LEFT, MARGIN_TOP);
            break;
    }
    if (loaded){
        mouseMotion();
    }
}
